from prettytable import PrettyTable
from auxi.modelling.business import *


def get(transactions, start_date, end_date, currency=""):
    table = PrettyTable(["Date", "Account", "Debit (dr)", "Credit (cr)"])
    table.align["Date"] = "l"
    table.align["Account"] = "l"
    table.align["Debit (dr)"] = "r"
    table.align["Credit (cr)"] = "r"
    table.float_format = ".2"
    amount_tot = 0
    for t in transactions:
        if t.date >= start_date and t.date <= end_date:
            table.add_row([t.date, t.debitAccountName, t.amount, ""])
            if t.creditAccount == "":
                table.add_row(["", "? (Look at Capital Loan's 'Interest' Financial transaction.", "", ""])
            else:
                table.add_row(["", t.creditAccountName, "", t.amount])
            amount_tot += t.amount
    table.add_row(["", "Total (dr)", amount_tot, ""])
    table.add_row(["", "Total (cr)", "", amount_tot])
    table.sort_key("Date")
    return table



"""auxi.modelling.business Package

The purpose of this package is to provide common classes and functions for the rest of the auxi.modelling.business namespace"""

__version__ = "0.1.0"

import auxi.modelling.accounting.financial
import auxi.modelling.accounting.stock
from auxi.modelling.business._business import *


from auxi.simulation.io.reader import *
from openpyxl import load_workbook
import pandas as pd
from auxi.simulation.path_engine import *
from auxi.simulation.investigation import *
from dateutil.parser import *


class Reader_Excel(Reader):
    def __init__(self, file_path):
        Reader.__init__(self, file_path)

    def update_scenario_from_file(self, investigation, scenario_name):
        workbook = load_workbook(self.file_path)

        scen_name = 'Parameters('+scenario_name+')'
        worksheet = workbook.get_sheet_by_name(scen_name)
        row_count = worksheet.max_row
        column_headers = ['Segment', 'Parameter',
                          'Description', 'Units', 'Value']
        dataframe = pd.DataFrame(columns=column_headers)

        # Clear all the current parameters in the scenario's segments.
        for seg_name, seg in investigation.scenarios[scenario_name].segments.items():
            seg.parameters.clear()

        # Update the scenario's segments with the parameters in the excel file.
        for count in range(2, row_count+1):
            segment_name = worksheet.cell(row=count, column=1).value
            segment_name = segment_name[segment_name.find("::")+2:]
            dataframe.loc[count, 'Segment'] = segment_name
            dataframe.loc[count, 'Parameter'] = worksheet.cell(row=count,
                                                               column=2).value
            dataframe.loc[count, 'Description'] = worksheet.cell(row=count,
                                                                 column=3).value
            dataframe.loc[count, 'Units'] = worksheet.cell(row=count,
                                                           column=4).value
            dataframe.loc[count, 'Value'] = worksheet.cell(row=count,
                                                           column=5).value

            excel_file_path = './' + self.file_path
            time_set_df = pd.read_excel(excel_file_path, scen_name)
            col_ix = 5
            number_of_settings = int((len(time_set_df.columns) - col_ix)/2)
            time_column_name = "DateTime_TimeStep_Interval"
            value_column_name = "Value"
            for j in range(1, number_of_settings+1):
                time_col = time_column_name+"."+str(j)
                val_col = value_column_name+"."+str(j)
                dataframe.loc[count, time_col] = worksheet.cell(
                    row=count, column=col_ix+(j*2)-1).value
                dataframe.loc[count, val_col] = worksheet.cell(
                    row=count, column=col_ix+(j*2)).value

            # Create the scenario in investigation
            if scenario_name in investigation.scenarios:
                scenario = investigation.scenarios[scenario_name]
            else:
                raise ValueError("The scenario '%s' does not exist in the investigation" % scenario_name)

            for param in dataframe.index:
                segment_name = dataframe.at[param, 'Segment']
                param_name = dataframe.at[param, 'Parameter']
                param_units = dataframe.at[param, 'Units']
                param_default_value = dataframe.at[param, 'Value']
                param_desc = dataframe.at[param, 'Description']
                # Don't create a new segment if the segment exists.
                if segment_name in scenario.segments:
                    segment = scenario[segment_name]
                else:
                    raise ValueError("The segment '%s' does not exist in the scenario '%s'." % (segment_name, scenario_name))

                #Create parameter in the investigation
                param_source = get_instance_value(segment.auxi_object,
                                                  param_name)
                new_param = segment.create_parameter(param_name,
                                                     param_source,
                                                     param_units,
                                                     param_default_value,
                                                     param_desc)

                #set_instance_value(segment.auxi_object,
                #                   param_name, param_default_value)

                for j in range(1, number_of_settings+1):
                    time_col = time_column_name+"."+str(j)
                    val_col = value_column_name+"."+str(j)
                    time_val = dataframe.at[param, time_col]
                    val = dataframe.at[param, val_col]
                    if time_val == time_val and val == val:
                        try:
                            dt = parse(time_val)
                            new_param.value_dict_dates[dt] = val
                        except AttributeError:
                            ix = eval(str(time_val))
                            if isinstance(ix, float):
                                new_param.value_dict_intervals[ix] = int(val)



from prettytable import PrettyTable
from auxi.modelling.business import *


def gl_accounts_to_table(table, prefix, account):
    table.add_row([prefix + account.name, account.description, account.accountNumber, account.type.name])
    for acc in account.generalLedgerAccountList:
        gl_accounts_to_table(table, "  " + prefix, acc)


def generalLedger_structure_to_string(generalLedger_structure):
    table = PrettyTable(["Name", "Description", "Number", "Type"])
    table.align = 'l'
    for acc in generalLedger_structure.generalLedgerAccountList:
        gl_accounts_to_table(table, "", acc)
    return table



from prettytable import PrettyTable
from auxi.modelling.business import *


def get(generalLedger, end_date, currency=""):
    transactions = generalLedger.transactionList
    ledger_struct = generalLedger.structure
    table = PrettyTable(["Assets", " ", "Liabilities and Equity", "", "  "])
    table.align["Assets"] = "l"
    table.align[" "] = "r"
    table.align["Liabilities and Equity"] = "l"
    table.align[""] = "r"
    table.align["  "] = "r"
    table.float_format = ".2"
    summedAssets = {}
    summedLiability = {}
    summedEquity = {}

    for t in transactions:
        if t.date <= end_date:
            cr_account_name = t.creditAccountName
            db_account_name = t.debitAccountName
            cr_account_type = ledger_struct.get_account(cr_account_name).type
            db_account_type = ledger_struct.get_account(db_account_name).type
            if cr_account_name != "":
                if cr_account_type == GeneralLedgerAccountType.Asset:
                    if cr_account_name in summedAssets: summedAssets[cr_account_name] -= t.amount
                    else: summedAssets[cr_account_name] = t.amount
                elif cr_account_type == GeneralLedgerAccountType.Liability:
                    if cr_account_name in summedLiability: summedLiability[cr_account_name] += t.amount
                    else: summedLiability[cr_account_name] = t.amount
                elif cr_account_type == GeneralLedgerAccountType.Equity:
                    if cr_account_name in summedEquity: summedEquity[cr_account_name] += t.amount
                    else: summedEquity[cr_account_name] = t.amount
            if db_account_name != "":
                if db_account_type == GeneralLedgerAccountType.Asset:
                    if db_account_name in summedAssets: summedAssets[db_account_name] += t.amount
                    else: summedAssets[db_account_name] = t.amount
                if db_account_type == GeneralLedgerAccountType.Liability:
                    if db_account_name in summedLiability: summedLiability[db_account_name] -= t.amount
                    else: summedLiability[db_account_name] = t.amount
                if db_account_type == GeneralLedgerAccountType.Equity:
                    if db_account_name in summedEquity: summedEquity[db_account_name] -= t.amount
                    else: summedEquity[db_account_name] = t.amount

    assets_count = len(summedAssets)
    lia_n_eq_count = len(summedLiability) + len(summedEquity) + 4 + 3

    assets_sum = sum(summedAssets.values())
    liabilities_sum = sum(summedLiability.values())
    equities_sum = sum(summedEquity.values())

    rows = []
    for i in range(0, max(assets_count, lia_n_eq_count)):
        rows.append(["", "", "", "", ""])
    ix = 0
    for entry in summedAssets:
        rows[ix][0] = entry
        rows[ix][1] = summedAssets[entry]
        ix += 1

    rows[0][2] = "Liabilities"
    rows[1][2] = "-----------"
    ix = 2
    for entry in summedLiability:
        rows[ix][2] = entry
        rows[ix][3] = summedLiability[entry]
        ix += 1

    rows[ix][2] = "Total liabilities"
    rows[ix][4] = liabilities_sum
    ix += 1
    rows[ix][2] = "-----------------"
    ix += 1
    rows[ix][2] = "Owners' Equity"
    ix += 1
    rows[ix][2] = "--------------"
    ix += 1
    for entry in summedEquity:
        rows[ix][2] = entry
        rows[ix][3] = summedEquity[entry]
        ix += 1
    rows[ix][2] = "Total owners' equities"
    rows[ix][4] = sum(summedEquity.values())

    rows.append(["-----", "--------", "----------------------", "", "--------"])
    rows.append(["Total", assets_sum, "Total", "", liabilities_sum + equities_sum])

    [table.add_row(row) for row in rows]

    return str(table)



from auxi.simulation.path_engine import *
from auxi.simulation.investigation import *


class Writer():
    def __init__(self, file_path):
        self.file_path = file_path

    def create_file_from_investigation(self, investigation):
        print("Abstract Method. Make use of a derived class.")



# -*- coding: utf-8 -*-
"""
Created on Tue Oct 13 16:30:02 2015

@author: Christoff Kok, Thabisani Nigel Phuthi
"""
__version__ = "0.0.0"

from auxi.modelling.business import NamedObject
from auxi.simulation.segment import *


class Scenario(NamedObject):
    """ A scenario container.

    :param name: The name of the scenario
    :param description: The description of the scenario. By default the scenario
        has no description, but one may be set by the user.

    Simulations may have different scenarios to be studied, where some or all
    parameters may need to be changed.
    Within a scenario there are segments. A segment corresponds to a specific submodel.

    To add a segment to a scenario, use the command::
        scenario['Scenario name'].add_segment('Segment name')
    """

    def __init__(self, name, description=""):
        NamedObject.__init__(self)
        self.name = name
        self.description = description
        self.segments = {}
        self._path = ""
        self.model_executionList = []

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, value):
        self._path = value

    def __getitem__(self, key):
        return self.segments[key]

    def add_segment(self, auxi_object, name):
        self.segments[name] = Segment(name)
        self.segments[name].path = self._path + '::' + name
        self.segments[name].auxi_object = auxi_object
        return self.segments[name]

    def apply_parameter_values(self, interval, date,
                               interval_type, interval_size):
        for seg_name, seg in self.segments.items():
            for param_path, param in seg.parameters.items():
                param.apply_value(seg.auxi_object, interval, date,
                                  interval_type, interval_size, self)

    def run(self):
        # Run the scenario steps
        # TODO: Add scenario steps
        for model in self.model_executionList:
            model.run_from_scenario(self)



# -*- coding: utf-8 -*-
"""
Created on Tue Oct 13 16:30:02 2015

@author: Christoff Kok, Thabisani Nigel Phuthi
"""
__version__ = "0.0.0"

from auxi.modelling.business import NamedObject
from auxi.simulation.parameter import *


class Segment(NamedObject):
    """ A segment in the database corresponding to a submodel

        :param name: The name of the segment.
        :param description: A description of the segment. (Default has no
                            description)

        The segment has containers for parameters (values to be used in
        simulationcalculations by a submodel) and results (values generated
        by submodels)

        To create a parameter with the segment use the command::
            Segment.create_parameter(name, source units, default_value,
                                     description)
        To add a parameter that has been created outside of this segement,
        use the command::
            Segment.add_parameter(parameter)
        To clone a separately created parameter to a segment use the command::
            Segment.clone_parameter(parameter)
        Similar to the parameter's add and create commands, a result may also
        be created or added respectively with the commands::
            Segment.create_result(name, date, units, description)
            Segment.add_result(result)
    """

    def __init__(self, name, description=""):
        NamedObject.__init__(self)
        self.name = name
        self.description = description
        self._path = None
        self.auxi_object = None
        self.parameters = {}
        self.results = {}

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, value):
        self._path = value

    def __getitem__(self, key):
        return self.parameters[key]

    def create_parameter(self, param_path, source, units, default_value=None,
                         description=""):
        parameter = Parameter(param_path, source, units,
                              default_value, description)
        parameter.path = param_path
        self.parameters[param_path] = parameter
        return self.parameters[param_path]

    def add_parameter(self, parameter):
        self.parameters[parameter.path] = parameter
        return parameter

    def clone_parameter(self, parameter):
        return self.parameters.append(parameter(parameter.path, parameter.units,
                                      parameter.default_value,
                                      parameter.description))



# -*- coding: utf-8 -*-
"""
Created on Tue Oct 14 10:19:02 2015

Contains functions to help create paths and access values specified by a path.

@author: Christoff Kok
"""

from datetime import datetime
from auxi.modelling.business import NamedObject


def generate_parameter_paths(root_object):
    return generate_paths(root_object, True)


def generate_result_paths(root_object):
    return generate_paths(root_object, False)


def get_class_properties(cls):
    result = []
    for p in cls.__dict__.keys():
        if p[:2] != '__' and (isinstance(getattr(cls, p), property)
                              or (p in vars(cls) and p != "is_param"
                                  and not callable(p))):
            result.append(p)
    return result


def generate_paths(root_object, exclude_non_settable_props, path=""):
    result = {}
    if path == "":
        path = root_object.name

    for a in get_class_properties(type(root_object)):
        if a == "":
            raise ValueError("Could not create a path to the '%s' object. \
at path '%s'." % (type(root_object), a))
        if not root_object.is_param(a):
            continue
        prop = getattr(root_object, a)
        if type(prop) in (int, float, bool, str, datetime):
            result[(path, root_object, a)] = prop
        elif hasattr(prop, '__iter__'):
            for item in prop:
                item_path = path + "//" + item.name
                for r_path, r_prop in generate_paths(item,
                                                     exclude_non_settable_props,
                                                     item_path).items():
                    result[r_path] = r_prop
        elif isinstance(prop, NamedObject):
            prop_path = path + "//" + prop.name
            for r_path, r_prop in generate_paths(prop,
                                                 exclude_non_settable_props,
                                                 prop_path).items():
                result[r_path] = r_prop
    return result


def get_instance_value(root_object, prop_name):
    return getattr(root_object, prop_name)


def set_instance_value(root_object, prop_name, value):
    if value is None:
        val = ""
    else:
        if isinstance(value, datetime):
            val = value
        else:
            val = eval(str(value))
    try:
        setattr(root_object, prop_name, val)
    except AttributeError:
        pass



# USAGE: you probably want 'setup.py install' - but execute 'setup.py --help'
# for all the details.

# from setuptools import setup, Extension, Command

from distutils.core import setup

# build the distribution
setup(name="auxi",
      version="0.1.0rc4",
      description="A toolkit to help metallurgical process engineers to rapidly do day-to-day calculations.",
      long_description=open('README.txt').read(),
      author="Ex Mente (Pty) Ltd",
      author_email="dev@ex-mente.co.za",
      maintainer="Ex Mente (Pty) Ltd",
      maintainer_email="dev@ex-mente.co.za",
      url="https://github.com/Ex-Mente/auxi.0",
      download_url="https://pypi.python.org/pypi/auxi",
      license=open('LICENSE.txt').read(),
      keywords="metallurgy,chemistry,modelling,simulation,thermochemistry,engineering,mass balance,energy balance",
      platforms="Ubunutu 14.04,Ubuntu 15.04",
      package_dir={'auxi': 'auxi'},
      packages=["auxi", "auxi.core",
                "auxi.modelling", "auxi.modelling.business", "auxi.modelling.accounting.financial", "auxi.modelling.accounting.stock", "auxi.simulation",
                "auxi.tools", "auxi.tools.chemistry"],
      package_data={'auxi.core': ['*.so*','*.so.1.54.0*', '*.a', '*.dll', '*.pyd'],
                    'auxi.modelling.accounting.financial': ['*.a', '*.so*', '*.dll', '*.pyd', '*_report.py'],
                    'auxi.modelling.accounting.stock': ['*.a', '*.so*', '*_report.py'],
                    'auxi.modelling.business': ['*.a', '*.so*', '*.dll', '*.pyd', '*_report.py'],
                    'auxi.simulation': ['*.py', r'io/*'],
                    'auxi.tools.chemistry': ['*.so*', '*.a', '*.dll', '*.pyd','*.so.1.54.0*', r'data/*'],
                    'auxi' : [r'*.txt']}
      )



"""auxi for Python Package

The purpose of this package is to make auxi's chemistry tools
available through a series of Python modules for easy access and use.

Python version:  3.4.0
auxi version: 0.1.0"""

__version__ = "0.1.0"

import os
import sys
from auxi.tools.chemistry import thermochemistry

module_path = os.path.dirname(sys.modules[__name__].__file__)
data_path = os.path.join(module_path, r"data")
#print(data_path)

thermochemistry.set_default_data_path(data_path)
#print(thermochemistry.get_data_file_path())
thermochemistry.load_data()



from prettytable import PrettyTable
from auxi.modelling.business import *


def get(generalLedger, start_date, end_date, currency=""):
    transactions = generalLedger.transactionList
    ledger_struct = generalLedger.structure
    summedIncome = {}
    summedExpenses = {}
    for t in transactions:
        if t.date >= start_date and t.date <= end_date:
            cr_account_name = t.creditAccountName
            db_account_name = t.debitAccountName
            cr_account_type = ledger_struct.get_account(cr_account_name).type
            db_account_type = ledger_struct.get_account(db_account_name).type
            if not t.isClosingCreditAccount and not t.isClosingDebitAccount and cr_account_name != "":
                if cr_account_type == GeneralLedgerAccountType.Revenue:
                    if cr_account_name in summedIncome: summedIncome[cr_account_name] += t.amount
                    else: summedIncome[cr_account_name] = t.amount
                elif cr_account_type == GeneralLedgerAccountType.Expense:
                    if cr_account_name in summedExpenses: summedExpenses[cr_account_name] -= t.amount
                    else: summedExpenses[cr_account_name] = t.amount
            if not t.isClosingCreditAccount and not t.isClosingDebitAccount and db_account_name != "":
                if db_account_type == GeneralLedgerAccountType.Revenue:
                    if db_account_name in summedIncome: summedIncome[db_account_name] -= t.amount
                    else: summedIncome[db_account_name] = t.amount
                elif db_account_type == GeneralLedgerAccountType.Expense:
                    if db_account_name in summedExpenses: summedExpenses[db_account_name] += t.amount
                    else: summedExpenses[db_account_name] = t.amount

    sum_incomes = sum(summedIncome.values())
    sum_expenses = sum(summedExpenses.values())
    net_income = sum_incomes - sum_expenses

    table = PrettyTable(["", "Debit", "Credit"])
    table.align[""] = "l"
    table.align["Debit"] = "r"
    table.align["Credit"] = "r"
    table.float_format = ".2"
    table.add_row(["Revenues", "", ""])
    table.add_row(["--------", "", ""])
    for entry in summedIncome:
        table.add_row([entry, summedIncome[entry], ""])
    table.add_row(["", "", "--------"])
    table.add_row(["GROSS REVENUES (including INTEREST income)", "", sum_incomes])
    table.add_row(["", "", "--------"])

    table.add_row(["Expenses", "", ""])
    table.add_row(["--------", "", ""])
    for entry in summedExpenses:
        table.add_row([entry, summedExpenses[entry], ""])
    table.add_row(["", "", "--------"])
    table.add_row(["TOTAL EXPENSES", "", "(%.2f)" % sum_expenses])
    table.add_row(["", "", "--------"])
    if net_income < 0:
        table.add_row(["NET INCOME", "", "(%.2f)" % abs(net_income)])
    else:
        table.add_row(["NET INCOME", "", abs(net_income)])
    return table
